$ifndef _SOFTWARE_I2C_INC_
$define _SOFTWARE_I2C_INC_
'
' This source code is provided 'as is' and any express or implied warranties, including,
' but not limited to, the implied warranties of merchantability and fitness for a particular
' purpose are disclaimed.
'
' In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,
' special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute
' goods or services; loss of use, data, or profits;  or business interruption) however caused and on any
' theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)
' arising in any way out of the use of this source code, even if advised of the possibility of such damage.
'
' Any views and conclusions contained in the source code and any documentation are those of the author's
' and should not be interpreted as representing official policies,
' either expressed or implied, of Crownhill Associates Ltd.
'
'----------------------------------------------------------------------------------------------
' Software I2C routines written in Proton BASIC
' The routines are for 8-bit or 16-bit devices
' Written by L.Johnson 06-02-14
'
' Setup the default pins for SDA and SCL
'
$ifndef I2C_SDA_Pin
    $define I2C_SDA_Pin PORTC.4
$endif

$ifndef I2C_SCL_Pin
    $define I2C_SCL_Pin PORTC.3
$endif

'----------------------------------------------------------------------------------------------
' Delay a fixed number of microseconds
'
$define I2C_hDelay() DelayUS 5

'----------------------------------------------------------------------------------------------

$if (_core = 24) || (_core = 33)        ' 16-bit device routines
'----------------------------------------------------------------------------------------------
' Write a single byte to the I2C bus
' Input     : pData holds the byte to write
' Output    : Returns holding the Acknowledge state from the bus
' Notes     : MSB first
'
Proc I2C_WriteByte(pData As Byte), Bit 
    Dim bIndex As Byte = 8
    Repeat
        If pData.7 = 1 Then
            Input I2C_SDA_Pin
        Else
            Output I2C_SDA_Pin
        EndIf
        I2C_hDelay()
        Input I2C_SCL_Pin
        I2C_hDelay()
        Output I2C_SCL_Pin
        I2C_hDelay()
        pData = pData << 1
        Dec bIndex
    Until bIndex = 0

    Input I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Result = I2C_SDA_Pin
    Output I2C_SCL_Pin
EndProc

'----------------------------------------------------------------------------------------------
' Shift in a byte from the I2C bus
' Input     : None
' Output    : The byte received
' Notes     : MSB first, sample before clock
'
Proc I2C_ReadByte(), Byte
    Dim bIndex As Byte = 8
    Result = 0
    Input I2C_SDA_Pin    
    Repeat
        Result = Result << 1
        '
        ' Wait for any clock stretching (This may benefit from a timeout being added)
        '
        Repeat
            Input I2C_SCL_Pin
        Until I2C_SCL_Pin = 1
        '
        ' Read the data
        '
        I2C_hDelay()
        Result.0 = I2C_SDA_Pin
        Output I2C_SCL_Pin
        I2C_hDelay()
        Dec bIndex
    Until bIndex = 0
EndProc

'----------------------------------------------------------------------------------------------
' Send an I2C bus start condition
' Input     : None
' Output    : None
' Notes     : A Start condition is high to low of SDA when SCL is high
'
Proc I2C_Start()
    Input I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SDA_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    I2C_hDelay()
EndProc

'----------------------------------------------------------------------------------------------
' Send an I2C bus restart condition
' Input     : None
' Output    : None
' Notes     : None
'
$define I2C_Restart() I2C_Start()

'----------------------------------------------------------------------------------------------
' Send an I2C bus stop condition
' Input     : None
' Output    : None
' Notes     : A Stop condition is low to high of SDA when SCL is high
'
Proc I2C_Stop()
    Output I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Input I2C_SDA_Pin
    I2C_hDelay()
EndProc

'----------------------------------------------------------------------------------------------
' Helper procedure for I2C acknowledge and Not acknowledge 
' Input     : None
' Output    : None
' Notes     : None
'
Proc I2C_hAckNack()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Input I2C_SDA_Pin
EndProc
'----------------------------------------------------------------------------------------------
' Initiate an I2C acknowledge
' Input     : None
' Output    : None
' Notes     : None
'
Proc I2C_Ack()
    Output I2C_SDA_Pin
    I2C_hAckNack()
EndProc

'----------------------------------------------------------------------------------------------
' Initiate an I2C Not acknowledge
' Input     : None
' Output    : None
' Notes     : None
'
Proc I2C_NAck()
    Input I2C_SDA_Pin
    I2C_hAckNack()
EndProc

'----------------------------------------------------------------------------------------------
' Initialise the I2C bus
'
_I2C_Main_:
    Low I2C_SCL_Pin
    Low I2C_SDA_Pin

$else                                   ' **** 8-Bit device routines ****
'
' Create variables used by the 8-bit device routines
'
    Dim I2C_bTemp As Byte System
    Dim I2C_bIndex As Byte System

'------------------------------------------------------------------------------------------------
 
    GoTo _I2C_Main_                     ' Jump over the subroutines and initialise the I2C bus

'----------------------------------------------------------------------------------------------
' Write a single byte to the I2C bus
' Input     : pData holds the byte to write
' Output    : Returns the Acknowledge state from the bus
' Notes     : MSB first
'
$define I2C_WriteByte(pData) '
    I2C_bTemp = pData        '
    _I2C_hWriteByte

_I2C_hWriteByte Macro-
    GoSub __I2C_hWriteByte
    Return_Bit I2C_SDA_Pin
Endm

#ifMacro- _I2C_hWriteByte
__I2C_hWriteByte:
    I2C_bIndex = 8
    Repeat
        If I2C_bTemp.7 = 1 Then
            Input I2C_SDA_Pin
        Else
            Output I2C_SDA_Pin
        EndIf
        I2C_hDelay()
        Input I2C_SCL_Pin
        I2C_hDelay()
        Output I2C_SCL_Pin
        I2C_hDelay()
        I2C_bTemp = I2C_bTemp << 1
        Dec I2C_bIndex
    Until I2C_bIndex = 0
'
' Close up
'
    Input I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Read a single byte from the I2C bus
' Input     : None
' Output    : The byte received
' Notes     : MSB first, sample before clock
'
$define I2C_ReadByte() _I2C_hReadByte

_I2C_hReadByte Macro-
    GoSub __I2C_hReadByte
    Return_Byte I2C_bTemp
Endm

#ifMacro- _I2C_hReadByte
__I2C_hReadByte:
    I2C_bTemp = 0
    Input I2C_SDA_Pin
    I2C_bIndex = 8
    Repeat
        I2C_bTemp = I2C_bTemp << 1
        '
        ' Wait for any clock stretching (This may benefit from a timeout being added)
        '
        Repeat
            Input I2C_SCL_Pin
        Until I2C_SCL_Pin = 1
        '
        ' Read the data
        '
        I2C_hDelay()
        I2C_bTemp.0 = I2C_SDA_Pin
        Output I2C_SCL_Pin
        I2C_hDelay()
        Dec I2C_bIndex
    Until I2C_bIndex = 0
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Send an I2C bus start condition
' Input     : None
' Output    : None
' Notes     : A Start condition is high to low of SDA when SCL is high
'
$define I2C_Start() _I2C_Start

_I2C_Start Macro-
    GoSub __I2C_Start
Endm

#ifMacro- _I2C_Start
__I2C_Start:
    Input I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SDA_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    I2C_hDelay()
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Send an I2C bus restart condition
' Input     : None
' Output    : None
' Notes     : None
'
$define I2C_Restart() I2C_Start()

'----------------------------------------------------------------------------------------------
' Send an I2C bus stop condition
' Input     : None
' Output    : None
' Notes     : A Stop condition is low to high of SDA when SCL is high
'
$define I2C_Stop() _I2C_Stop

_I2C_Stop Macro-
    GoSub __I2C_Stop
Endm

#ifMacro- _I2C_Stop
__I2C_Stop:
    Output I2C_SDA_Pin
    I2C_hDelay()
    Input I2C_SCL_Pin
    I2C_hDelay()
    Input I2C_SDA_Pin
    I2C_hDelay()
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Initiate an I2C acknowledge
' Input     : None
' Output    : None
' Notes     : None
'
$define I2C_Ack() _I2C_Ack

_I2C_Ack Macro-
    GoSub __I2C_Ack
Endm

#ifMacro- _I2C_Ack
__I2C_Ack:
    Output I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Input I2C_SDA_Pin
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Initiate an I2C Not acknowledge
' Input     : None
' Output    : None
' Notes     : None
'
$define I2C_NAck() _I2C_NAck

_I2C_NAck Macro-
    GoSub __I2C_NAck
Endm

#ifMacro- _I2C_NAck
__I2C_NAck:
    Input I2C_SDA_Pin
    Input I2C_SCL_Pin
    I2C_hDelay()
    Output I2C_SCL_Pin
    Input I2C_SDA_Pin
    Return
#endIfMacro-

'----------------------------------------------------------------------------------------------
' Initialise the I2C bus
'
_I2C_Main_:
    Low I2C_SCL_Pin
    Low I2C_SDA_Pin

$endif  ' _core
$endif  ' _SOFTWARE_I2C_INC_

